unit UserFunctions;

interface

{$I ..\Parser.inc}
{$I ..\Demo.inc}

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Buttons, Grids, Menus, ExtCtrls;

type
  TUserFunctionsForm = class(TForm)
    PopupMenu1: TPopupMenu;
    ShowCalculator1: TMenuItem;
    Grid: TStringGrid;
    Panel1: TPanel;
    FillButton: TButton;
    Panel2: TPanel;
    AddFormulaButton: TBitBtn;
    InsertFormulaButton: TBitBtn;
    DeleteFormulaButton: TBitBtn;
    ClearFormulaButton: TBitBtn;
    MoveFormulaUpButton: TBitBtn;
    MoveFormulaDownButton: TBitBtn;
    Panel3: TPanel;
    HelpButton: TButton;
    ApplyButton: TButton;
    CancelButton: TButton;
    OkButton: TButton;
    procedure AddFormulaButtonClick(Sender: TObject);
    procedure ShowCalculator1Click(Sender: TObject);
    procedure FillButtonClick(Sender: TObject);
    procedure ApplyButtonClick(Sender: TObject);
    procedure OkButtonClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure DeleteFormulaButtonClick(Sender: TObject);
    procedure InsertFormulaButtonClick(Sender: TObject);
    procedure MoveFormulaUpButtonClick(Sender: TObject);
    procedure MoveFormulaDownButtonClick(Sender: TObject);
    procedure CancelButtonClick(Sender: TObject);
    procedure FormResize(Sender: TObject);
  private
    { Dclarations prives }
    procedure DeleteRow(index: integer);
    procedure InsertRow(index: integer);
    procedure MoveRowUp(index: integer);
    procedure MoveRowDown(index: integer);
    function Check(SayIfOk: boolean): boolean;
    procedure Clean;
  public
    { Dclarations publiques }
    procedure DebugFill;
  end;

var
  UserFunctionsForm: TUserFunctionsForm;

const
  SFormula = 'Formula, ex. g(h%s y) := h*ln(y)';

implementation

{$R *.dfm}

uses
  ParserTypes, ParserConsts, MathParser, Targets, Calc, ShowTree;

procedure TUserFunctionsForm.ShowCalculator1Click(Sender: TObject);
begin
  CalcForm.ShowModal;
end;

procedure TUserFunctionsForm.AddFormulaButtonClick(Sender: TObject);
var
  s: string;
begin
  with Grid do begin
    if trim(Cells[0, RowCount-1]) <> '' then begin
      RowCount := RowCount + 1;
      Cells[0, RowCount-1] := '';
    end;
    Row := RowCount-1;
  end;
  Grid.SetFocus;
end;

procedure TUserFunctionsForm.Clean;
var
  r: integer;
begin
  for r := Grid.RowCount-1 downto 1 do
    if trim(grid.Cells[0, r]) = '' then
      DeleteRow(r)
end;

type THack = class(TStringGrid);

function TUserFunctionsForm.Check(SayIfOk: boolean): boolean;
var
  r: integer;
  s: string;
  msg: string;
  allok: boolean;
  selstart: integer;
  sellength: integer;
begin
  result := true;
  ExternalTargets.Clear;
  Grid.EditorMode := false;
  msg := '';
  for r := 1 to Grid.RowCount - 1 do begin
    s := trim(Grid.Cells[0, r]);
    if s <> '' then begin
      try
        ParseAssign(s);
      Except
        On E: Exception do begin
          result := false;
          Grid.Row := r;
          if (E is EMathParserErr) then with EMathParserErr(E) do begin
            selstart := TokenStart-1;
            sellength := TokenLength;
            msg := ParseMessage(ParserError)
          end
          else begin
            selstart := 0;
            sellength := length(Grid.Cells[0, r]);
            msg := E.Message;
          end;
        end;
      end;
    end;
    if not result then
      break;
  end;
  Grid.SetFocus;
  if not result then begin
    MessageDlg(msg, mtError, [mbOK], 0);
    Grid.SetFocus;
    Grid.EditorMode := true;
    THack(Grid).InplaceEditor.SelStart := selstart;
    THack(Grid).InplaceEditor.SelLength := sellength;
  end
  else if SayIfOk then begin
    MessageDlg('All definitions correct', mtInformation, [mbOK], 0);
  end;
end;

procedure TUserFunctionsForm.OkButtonClick(Sender: TObject);
begin
  if Check(false) then begin
    Clean;
    ModalResult := mrOk;
  end;
end;

procedure TUserFunctionsForm.ApplyButtonClick(Sender: TObject);
begin
  Check(true);
end;

procedure TUserFunctionsForm.FillButtonClick(Sender: TObject);
begin
  DebugFill;
end;

procedure TUserFunctionsForm.CancelButtonClick(Sender: TObject);
var
  r: integer;
begin
  ExternalTargets.Clear;
  for r := 1 to Grid.RowCount - 1 do begin
    Grid.Cells[0, r] := trim(Grid.Cells[0, r]);
    try
      ParseAssign(Grid.Cells[0, r]);
    Except
      Grid.Cells[0, r] := '';
    end;
  end;
  Clean;
  ModalResult := mrCancel;
end;

procedure TUserFunctionsForm.DebugFill;
var
  c: char;
  r: integer;
begin
  with Grid do begin
    RowCount := 9;
    r := 1;
    for c := 'a' to 'd' do begin
      Cells[0,r] := Format('%s := %d', [c, ord(c)]);
      inc(r);
    end;
    Cells[0, r] := 'f(x) := 2*x';
    inc(r);
    Cells[0, r] := 'm := f(2)';
    inc(r);
    Cells[0, r] := 'k := 8';
    inc(r);
    Cells[0, r] := '';
  end;
  Grid.SetFocus;
end;

procedure TUserFunctionsForm.DeleteFormulaButtonClick(Sender: TObject);
begin
  if Grid.RowCount = 2 then
    Grid.Cells[0,1] := ''
  else
    DeleteRow(Grid.Row);
  Grid.SetFocus;
end;

procedure TUserFunctionsForm.DeleteRow(index: integer);
var
  r: integer;
begin
  for r := index to Grid.RowCount-2 do
    Grid.Cells[0, r] := Grid.Cells[0, r+1];
  Grid.RowCount := Grid.RowCount-1;
end;

procedure TUserFunctionsForm.InsertFormulaButtonClick(Sender: TObject);
begin
  InsertRow(Grid.Row);
  Grid.SetFocus;
end;

procedure TUserFunctionsForm.InsertRow(index: integer);
var
  r: integer;
begin
  Grid.RowCount := Grid.RowCount+1;
  for r := Grid.RowCount-1 downto index + 1 do
    Grid.Cells[0, r] := Grid.Cells[0, r-1];
  Grid.Cells[0, index] := '';
end;

procedure TUserFunctionsForm.MoveFormulaDownButtonClick(Sender: TObject);
begin
  if Grid.Row < Grid.RowCount-1 then begin
    MoveRowDown(Grid.Row);
    Grid.Row := Grid.Row+1;
  end;
  Grid.SetFocus;
end;

procedure TUserFunctionsForm.MoveFormulaUpButtonClick(Sender: TObject);
begin
  if Grid.Row > 1 then begin
    MoveRowUp(Grid.Row);
    Grid.Row := Grid.Row-1;
  end;
  Grid.SetFocus;
end;

procedure TUserFunctionsForm.MoveRowDown(index: integer);
var
  s: string;
begin
  s := Grid.Cells[0, index+1];
  Grid.Cells[0,index+1] := Grid.Cells[0,Index];
  Grid.Cells[0,index] := s;
end;

procedure TUserFunctionsForm.MoveRowUp(index: integer);
var
  s: string;
begin
  s := Grid.Cells[0, index-1];
  Grid.Cells[0,index-1] := Grid.Cells[0,Index];
  Grid.Cells[0,index] := s;
end;

procedure TUserFunctionsForm.FormCreate(Sender: TObject);
begin
  Grid.Cells[0,0] := Format(SFormula, [ParserListChar]);
  {$IFDEF HAS_ANCHORS}
    Grid.Width := clientwidth - panel2.width - 2*Grid.left;
    Grid.height := clientheight - panel1.height - 2*Grid.Top;
    Grid.Anchors := [akLeft, akTop, akRight, akBottom];
  {$ENDIF}
end;

procedure TUserFunctionsForm.FormResize(Sender: TObject);
begin
  {$IFNDEF HAS_ANCHORS}
    Grid.Width := clientwidth - panel2.width - 2*Grid.left;
    Grid.height := clientheight - panel1.height - 2*Grid.Top;
  {$ENDIF}
end;

end.
