(***********************************************************************
 *
 * Parser.inc version 1.3
 *
 * Copyright (c) 2013, 2014 Michel Deslierres
 * All rights reserved. Tous droits rservs.
 *
 * Read full license in the file license.txt
 * Lire la licence au complet dans le fichier license.txt
 *
 ***********************************************************************
 *
 * Changements / Changes
 *
 * - version 1.3
 *     Added CAN_REDEFINE_BUILTIN_FUNCTION (see note below)
 *
 * - version 1.2
 *     Added HAS_SYSCONST for backward compatibility with older
 *        versions of Delphi.
 *     Added HAS_LISTSEPARATOR for backward compatibility with older
 *        versions of Delphi
 *     Added FIX_ARCCSSH (see note below)
 *
 * - version 1.1
 *     Added USE_MATH_UNIT, EXPONENT_RIGHT_ASSOCIATIVE and DELPHI_DIV_MOD
 *
 *************************************************************************)

{: File with program wide compiler defines}

{$IFNDEF PARSER_INC}
  {$DEFINE PARSER_INC}

    {Define this compiler directive to allow user defined function to have
	 the same name as built-in functionsw which in effect redefines them}
  {.$DEFINE CAN_REDEFINE_BUILTIN_FUNCTION}
  
   { Define this compiler directive if SysUtils has the locale
     variable ListSeparator. Delphi version 2 does not!}
  {$DEFINE HAS_LISTSEPARATOR}

  { Define this compiler directive if SysConst unit is included
    with the version of Delphi used. Delphi version 2 does not!}
  {$DEFINE HAS_SYSCONST}

  { Define this directive to use the Math unit included with the
    versions of Delphi being used if available and if it has all
    needed functions. Otherwise the functions in ParserMath.pas will
    be used. Note that the implementation in Math.pas is probably
    better than in ParserMath.pas.}
  {.$DEFINE USE_MATH_UNIT}

  {$IFDEF USE_MATH_UNIT}
    {}
    { ArcCscH is incorrectly implemented in Math.pas included in
      versions of Delphi prior to Rad Studio 8.0 (Delphi XE).
      ParserMath.pas has a corrected version of the function which
      explains the $IFDEF USE_MATH_UNIT directive.}
    {$DEFINE FIX_ARCCSCH}
    {}
  {$ENDIF}

  { Define this directive to obtain the same results as with Delphi when
    performing div and mod functions (':' and '%'). If not the directive
    is not defined, Euclidean definitions will be used.}
  {$DEFINE DELPHI_DIV_MOD}

  { This directive means that 2^3^4 will be calculated as 2^(3^4). This is
    the more natural rule. If the directive is not defined then
    exponentiation will be left associative which means that
    2^3^4 will be calculated as (2^3)^4. It may be how Excel and
    OpenOffice.Calc do it but having 2^3^4 equal to 2^(3*4) is strange.}
  {$DEFINE EXPONENT_RIGHT_ASSOCIATIVE}

  { Define this compiler directive if the compiler recognizes
    the keyword ResourceString. If the directive is not defined
    strings will be declared as Const. Delphi verion 2 does not!}
  {.$DEFINE HAS_RESOURCESTRING}

  { Define this compiler directive to have single pass parsers
    accumulate information in a "journal" that can be displayed
    in a memo. Undefine this for release versions.}
  {$DEFINE DUMP}

{$ENDIF}
