(***********************************************************************
 *
 * ParserConsts version 1.2
 *
 * Copyright (c) 2013, 2104 Michel Deslierres
 * All rights reserved. Tous droits rservs.
 *
 * Read full license in the file license.txt
 * Lire la licence au complet dans le fichier license.txt
 *
 ***********************************************************************
 *
 * Changements / Changes
 *
 * - version 1.2
 *     Added new parser error messages related to assignment
 *     statements.
 *     Removed listchar parameter in ParseMessage to make
 *     internationalization simpler.
 *
 * - version 1.1
 *     Modified to handle three new parser error messages
 *     related to call of internal functions:
 *     peExpectedSeparator, peExpectedLeftPar, and
 *     peUnknownIdentifier.
 *
 ************************************************************************)

{: Unit with string contants for parser.}
unit ParserConsts;

{$IFNDEF PARSER_INC}
  {$I Parser.inc}
{$ENDIF}

interface

uses
  ParserTypes;

{$IFDEF HAS_RESOURCESTRING}
resourcestring
{$ELSE}
const
{$ENDIF}
  SParseError = 'Math parser error';
  SParameterCountFixed = 'The function has a fixed number of parameters';
  SUnknown = '<unknown>';

  {Preparation for internationalization using dxgettext}
function _(const value: string): string;

  {Returns an errror message based on the specified TParserError. The
   parser's list char must also be given.}
function ParseMessage(value: TParserError): string;

implementation

uses
  SysUtils;

function _(const value: string): string;
begin
  result := value;
end;

function ParseMessage(value: TParserError): string;
begin
  case value of
    peNoError:  result := _('no parser error');
    peUnexpectedEOS: result := _('unexpected end of source');
    peExpectedEOS: result := _('expected end of source');
    peInvalidNumber: result := _('invalid number');
    peUnexpectedElement: result := _('unexpected element');
    peExpectedRightPar: result := _('expected ")"');
    peExpectedSeparator: result := _('expected list separator');
    peExpectedLeftPar: result := _('expected "("');
    peUnknownIdentifier: result := _('unknown function or constant');
    peTooFewParams: result := _('missing parameters');
    peTooManyParams: result := _('too many parameters');
    peExpectedIdentifier: result := _('expected an identifier');
    peExpectedAssign: result := _('expected ":="');
    peDuplicateIdentifier: result := _('function or constant already defined');
    {$IFNDEF CAN_REDEFINE_BUILTIN_FUNCTION}
    peIdentIsBuiltinName: result := _('function or constant already defined');
    {$ENDIF}
    peExpectedFormalParameter: result := _('expected a function argument');
    peInvalidFormalParameterName: result := _('invalid parameter name');
    peDuplicateFormalParameterName: result := _('parameter name already used');
    peExpectedSeparatorOrRightPar: result := _('expected a list separator or ")"');
  else
    result := Format(_('unknown error #%d') , [ord(value)])
  end;
end;

end.
