﻿unit TestModDiv;

interface

uses
  TestFramework, SysUtils, Classes, ParserMath;

type
  TTestModsDivs = class(TTestCase)
  published
    procedure TestPascalDiv;
    procedure TestDivE;
    procedure TestDivT;
  end;

implementation

{ TTestModsDivs }

(* Reference:

Division and Modulus for Computer Scientists
DAAN LEIJEN

   D = dividend
   d = divisor

   q = quotient
   r = remainder

   T - truncation method
   E - Euclidean method

  (D; d)    (qT ; rT )   (qE; rE)
  (+8; +3)  (+2; +2)     (+2; +2)
  (+8; -3)  (-2; +2)     (-2; +2)
  (-8; +3)  (-2;-2)      (-3; +1)
  (-8; -3)  (+2;-2)      (+3; +1)
  (+1; +2)  ( 0; +1)     ( 0; +1)
  (+1; -2)  ( 0; +1)     ( 0; +1)
  (-1; +2)  ( 0;-1)      (-1; +1)
  (-1; -2)  ( 0;-1)      (+1; +1)
*)

procedure TTestModsDivs.TestDivT;
begin
  CheckEquals(2, DivT(8, 3));
  CheckEquals(2, ModT(8, 3));

  CheckEquals(-2, DivT(8, -3));
  CheckEquals(2, ModT(8, -3));

  CheckEquals(-2, DivT(-8, 3));
  CheckEquals(-2, ModT(-8, 3));

  CheckEquals(2, DivT(-8, -3));
  CheckEquals(-2, ModT(-8, -3));

  CheckEquals(0, DivT(1, 2));
  CheckEquals(1, ModT(1, 2));

  CheckEquals(0, DivT(1, -2));
  CheckEquals(1, ModT(1, -2));

  CheckEquals(0, DivT(-1, 2));
  CheckEquals(-1, ModT(-1, 2));

  CheckEquals(0, DivT(-1, -2));
  CheckEquals(-1, ModT(-1, -2));

  CheckEquals(2, DivT(8.2, 4.1));
  CheckEquals(0, ModT(8.2, 4.1));
end;


procedure TTestModsDivs.TestDivE;
begin
  CheckEquals(2, DivE(8, 3));
  CheckEquals(2, ModE(8, 3));

  CheckEquals(-2, DivE(8, -3));
  CheckEquals(2, ModE(8, -3));

  CheckEquals(-3, DivE(-8, 3));
  CheckEquals(1, ModE(-8, 3));

  CheckEquals(3, DivE(-8, -3));
  CheckEquals(1, ModE(-8, -3));

  CheckEquals(0, DivE(1, 2));
  CheckEquals(1, ModE(1, 2));

  CheckEquals(0, DivE(1, -2));
  CheckEquals(1, ModE(1, -2));

  CheckEquals(-1, DivE(-1, 2));
  CheckEquals(1, ModE(-1, 2));

  CheckEquals(1, DivE(-1, -2));
  CheckEquals(1, ModE(-1, -2));

  CheckEquals(2, DivE(8.2, 4.1));
  CheckEquals(0, ModE(8.2, 4.1));
end;

procedure TTestModsDivs.TestPascalDiv;
begin
  CheckEquals(2, 8 div 3);
  CheckEquals(2, 8 mod 3);

  CheckEquals(-2, 8 div -3);
  CheckEquals(2, 8 mod -3);

  CheckEquals(-2, -8 div 3);
  CheckEquals(-2, -8 mod 3);

  CheckEquals(2, -8 div -3);
  CheckEquals(-2, -8 mod -3);

  CheckEquals(0, 1 div 2);
  CheckEquals(1, 1 mod 2);

  CheckEquals(0, 1 div -2);
  CheckEquals(1, 1 mod -2);

  CheckEquals(0, -1 div 2);
  CheckEquals(-1, -1 mod 2);

  CheckEquals(0, -1 div -2);
  CheckEquals(-1, -1 mod -2);

  //CheckEquals(2, 8.2 div 4.1); will not parse, div
  //CheckEquals(0, 8.2 mod 4.1); and mod operands must be integers
end;

initialization
  // Recenser tous les cas de test avec le lanceur de test
  RegisterTest(TTestModsDivs.Suite);
end.
